package gssim.db.mysql;

import gssim.db.dao.data.Queues;
import gssim.db.find.Finder;
import gssim.db.utils.PreparedStatementAdapter;
import gssim.db.utils.ResultSetAdapter;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.LogFactory;

import com.mysql.jdbc.NotImplemented;

public class QueuesDAO extends AbstractDAO implements gssim.db.dao.QueuesDAO {

	protected QueuesDAO(Connection conn) throws SQLException {
		super(conn);
		log = LogFactory.getLog(QueuesDAO.class);
		setRelationName("queues");
	}

	@Override
	protected PreparedStatementAdapter createDeleteStmt() throws SQLException {
		return null;
	}

	@Override
	protected PreparedStatementAdapter createFindStmt() throws SQLException {
		String sql = "SELECT * FROM queues WHERE"
				+ " (fk_resource_id = ? OR ISNULL(?))";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createInsertStmt() throws SQLException {
		String sql = "INSERT INTO queues VALUES(?, ?, ?)";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));

		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createSelectStmt() throws SQLException {
		String sql = "SELECT * FROM queues WHERE fk_resource_id = ? AND name = ?";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createUpdateStmt() throws SQLException {
		return null;
	}

	@Override
	public boolean delete(Queues obj, boolean commit) throws SQLException {
		throw new NotImplemented();
	}

	@Override
	public void fillObjectFromResultSet(Queues obj, ResultSet result)
			throws SQLException {
		throw new NotImplemented();

	}

	@Override
	public int fillStatementWithObject(PreparedStatement stmt, Queues obj,
			int offset) throws SQLException {
		throw new NotImplemented();
	}

	@Override
	public List<Queues> find(Finder finder) throws SQLException {

		PreparedStatementAdapter stmt = getFindStmt();
		List<Queues> list = new ArrayList<Queues>();

		stmt.setInt(1, finder.getInteger("fkResourceId"));
		stmt.setInt(2, finder.getInteger("fkResourceId"));

		ResultSetAdapter result = new ResultSetAdapter(stmt.executeQuery());

		while (result.next()) {
			Queues obj = new Queues();

			Integer fkResourceId = result.getIntObj("fk_resource_id");
			String name = result.getString("name");
			Float avgLength = result.getFloatObj("avg_length");

			obj.setFkResourceId(fkResourceId);
			obj.setName(name);
			obj.setAvgLength(avgLength);

			list.add(obj);
		}

		return list;
	}

	@Override
	public int insert(Queues obj, boolean commit) throws SQLException {

		PreparedStatementAdapter stmt = getInsertStmt();

		stmt.setInt(1, obj.getFkResourceId());
		stmt.setString(2, obj.getName());
		stmt.setFloat(3, obj.getAvgLength());

		int result = stmt.executeUpdate();

		if (result == 1) {
			if (commit)
				commit();
			logSuccess("insert");
		} else
			logError("insert", result, obj);

		return result;
	}

	@Override
	public int[] insertBatch(List<Queues> objList, boolean commit)
			throws SQLException {

		PreparedStatementAdapter stmt = getInsertStmt();

		for (int i = 0; i < objList.size(); i++) {
			stmt.setInt(1, objList.get(i).getFkResourceId());
			stmt.setString(2, objList.get(i).getName());
			stmt.setFloat(3, objList.get(i).getAvgLength());
			stmt.addBatch();
		}

		int[] result = stmt.executeBatch();

		stmt.clearBatch();

		int j = 0;

		while (j < result.length)
			if (result[j] == 1)
				j++;

		if (j == result.length) {
			if (commit)
				commit();

			logSuccess("batch insert", j);

		} else
			logError("batch insert", j);

		return result;
	}

	@Override
	public Queues select(Queues obj) throws SQLException {

		PreparedStatementAdapter stmt = getSelectStmt();
		stmt.setInt(1, obj.getFkResourceId());
		stmt.setString(2, obj.getName());
		ResultSetAdapter result = new ResultSetAdapter(stmt.executeQuery());

		result.next();
		boolean first = result.isFirst();
		boolean last = result.isLast();

		if (first && last) {

			obj.setAvgLength(result.getFloatObj("avg_length"));
			return obj;
		}

		// no reservation found with specified task_id and unit_id
		if (!first && !last) {
			return null;
		}

		// no idea how this may happen (only if id is not primary key)
		throw new SQLException("Select operation in relation "
				+ getRelationName() + " returned more than one record");
	}

	@Override
	public boolean update(Queues obj, boolean commit) throws SQLException {
		throw new NotImplemented();
	}

}